USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_Microsoft_IAMUniversalInventory' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_Microsoft_IAMUniversalInventory]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 10.01.2018
-- Last update: 11.01.2018
-- Description:	Export RayVentory Data to
--              Microsoft IAM Universal Inventory
-- =============================================
-- Parameter:
--   @UniversalDB       Database Name of Universal Inventory (UI_{ProjectName})
--                      
--   @DatabaseSourceID  Define a SourceID on witch the Data will be Imported
--                      If you will Update Data, use the same as on the first Import.
-- =============================================
-- Change Log:
-- 11.01.2018 Add better User and Logon Information
CREATE PROCEDURE [dbo].[csp_Microsoft_IAMUniversalInventory] (
  @UniversalDB      NVARCHAR(256),
  @DataSourceID     NVARCHAR(32))
AS
BEGIN
--  DECLARE @UniversalDB      NVARCHAR(256)  = 'UI_test3'
--  DECLARE @DataSourceID     NVARCHAR(32)   = 'RayVentory_1'

  DECLARE @lastErrorMsg     NVARCHAR(4000) = null
  DECLARE @HyperVWithSerial BIT            = 0
  DECLARE @sSQL             VARCHAR(MAX)

  DECLARE @sTmpRandom       VARCHAR(10)    = CAST(ROUND((10000-1) * RAND(), 0) AS VARCHAR(10))
  DECLARE @sTmpFQDNWork     VARCHAR(256)   = '##tmpDeviceFQDN_Work_' + @sTmpRandom
  DECLARE @sTmpFQDN         VARCHAR(256)   = '##tmpDeviceFQDN_' + @sTmpRandom
  DECLARE @sTmpBIOS         VARCHAR(256)   = '##tmpDeviceBIOS_' + @sTmpRandom
  DECLARE @sTmpBaseBoard    VARCHAR(256)   = '##tmpBaseBoard_' + @sTmpRandom
  DECLARE @sTmpOS           VARCHAR(256)   = '##tmpDeviceOS_' + @sTmpRandom
  DECLARE @sTmpTblVms       VARCHAR(256)   = '##tmpTblVms_' + @sTmpRandom
  /********************************************************************************************
   * Cleanup Data Source
   ********************************************************************************************
   * Description: Cleanup the Import Data for the DataSourece @DataSourceID
   ********************************************************************************************
   * Created:     12.12.2018
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/
  SET @sSQL = '
    DECLARE @lastErrorMsg NVARCHAR(4000) = ' + IIF(@lastErrorMsg IS NULL, 'NULL',  '''' + @lastErrorMsg + '''') + '
    EXEC ' + @UniversalDB + '.[in].[sp_CleanDataSource] ''' + @DataSourceID + ''', @lastErrorMsg
    SELECT @lastErrorMsg AS [@lastErrorMsg]'
  EXECUTE (@sSQL)

  /********************************************************************************************
   * FQDN Information
   ********************************************************************************************
   * Description: Get FQDN Information for all Imports in Temptable
   ********************************************************************************************
   * Created:     12.12.2018
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/
  SET @sSQL = '
    IF OBJECT_ID(''tempdb..' + @sTmpFQDNWork + ''') IS NOT NULL
      DROP TABLE ' + @sTmpFQDNWork
  EXECUTE (@sSQL)

  SET @sSQL = '
  CREATE TABLE ' + @sTmpFQDNWork + ' (
    computerid              BIGINT,
    computercn              NVARCHAR(255) COLLATE database_default,
	fqdn                    NVARCHAR(255) COLLATE database_default,
    InventoryDate           DATETIME)'
  EXECUTE (@sSQL)

  SET @sSQL = '
  INSERT INTO ' + @sTmpFQDNWork + ' (computerid, computercn, fqdn, InventoryDate)
  SELECT tmp.ComputerID, tmp.ComputerCN, tmp.FQDN, ir.HWDate AS InventoryDate
  FROM (SELECT
            c.ComputerID,
            COALESCE(opCSName.Value, c.ComputerCN) AS ComputerCN,
            CASE
              WHEN baseDom.Value IS NOT NULL THEN COALESCE(opCSName.Value, c.ComputerCN) + ''.'' + baseDom.Value
              WHEN CHARINDEX(''.'', c.ComputerCN) > 0 THEN c.ComputerCN
              WHEN CHARINDEX('','', d.DN) > 1 THEN c.ComputerCN + REPLACE(ISNULL(''.'' + CASE WHEN dbo.CIMString(o.RDN) NOT LIKE ''%[-z0-9._-]%'' THEN REPLACE(REPLACE(dbo.CIMString(o.RDN), ''CN='', ''''), ''OU='', '''') END , '''') + ''.''  + REPLACE(d.DN, ''DC='', ''''), '','', ''.'')
              WHEN dbo.CIMString(nd.DNSFullName) IS NOT NULL THEN nd.DNSFullName
              WHEN dns.DNSSuffix IS NOT NULL THEN c.ComputerCN + ''.'' + dns.DNSSuffix
              ELSE COALESCE(opCSName.Value, c.ComputerCN)
            END FQDN
          FROM Computer c WITH (NOLOCK)
            LEFT JOIN Organization o WITH (NOLOCK)
              ON c.ComputerOUID = o.OrganizationID
            LEFT JOIN Domain d WITH (NOLOCK)
              ON o.DomainID = d.OrganizationID
            LEFT JOIN (SELECT ComputerID, MAX(DNSFullName) AS DNSFullName FROM NetworkDevice nd WITH (NOLOCK) GROUP BY ComputerID) nd
              ON c.ComputerID = nd.ComputerID
            LEFT JOIN f_HardwareProperties(''CIM_UnitaryComputerSystem'') baseDom
              ON c.ComputerID = baseDom.ComputerID
                AND Property = ''Domain''
            LEFT JOIN (SELECT
                           nic.ComputerID,
                           MAX(dns.Value) AS DNSSuffix
                         FROM f_HardwareProperties(''Win32_NetworkAdapterConfiguration'') nic
                           LEFT JOIN f_HardwareValuesByPropertyName(''DNSDomain'') dns
                             ON nic.HardwareObjectID = dns.HardwareObjectID
                         WHERE nic.Property = ''IPEnabled'' AND nic.Value = ''TRUE'' AND dns.Value IS NOT NULL
                         GROUP BY nic.ComputerID) dns
              ON c.ComputerID = dns.ComputerID
-- 08.11.2018 p.a. Fix for ComputerCN with IPAddress and Inventories
            LEFT JOIN f_HardwareProperties(''CIM_OperatingSystem'') opCSName
              ON opCSName.Property = ''CSNAME''
                AND c.ComputerID = opCSName.ComputerID
-- 05.12.2018 p.a. Fix Problem of space in FQDN and Length over 255 Chars
       ) tmp
    LEFT JOIN InventoryReport ir
      ON tmp.ComputerID = ir.ComputerID
  WHERE LEN(FQDN) <= 255 AND FQDN NOT LIKE ''%[^a-z0-9._-]%'''
  EXECUTE (@sSQL)

  SET @sSQL = '
  IF OBJECT_ID(''tempdb..' + @sTmpFQDN + ''') IS NOT NULL
    DROP TABLE ' + @sTmpFQDN 
  EXECUTE (@sSQL)

  SET @sSQL = '
  CREATE TABLE ' + @sTmpFQDN + ' (
    computerid              BIGINT,
    computercn              NVARCHAR(255) COLLATE database_default,
	fqdn                    NVARCHAR(255) COLLATE database_default,
    InventoryDate           DATETIME)'
  EXECUTE (@sSQL)

  SET @sSQL = '
  INSERT INTO ' + @sTmpFQDN + '
  SELECT
      ComputerID,
	  ComputerCN,
	  fqdn,
	  InventoryDate
    FROM ' + @sTmpFQDNWork + ' d
	WHERE d.InventoryDate IN (SELECT MAX(InventoryDate) FROM ' + @sTmpFQDNWork + ' t WHERE t.fqdn = d.fqdn GROUP BY t.fqdn)'
  EXECUTE (@sSQL)

  SET @sSQL = '
  DELETE d
    FROM ' + @sTmpFQDN + ' d
    WHERE d.computerid IN (SELECT MAX(ComputerID) FROM ' + @sTmpFQDN + ' t GROUP BY t.fqdn HAVING COUNT(t.fqdn) > 1)'
  EXECUTE (@sSQL)

  SET @sSQL = '
  DROP TABLE ' + @sTmpFQDNWork
  EXECUTE (@sSQL)

  /********************************************************************************************
   * Device
   ********************************************************************************************
   * Description: Get all Devices
   *                - Windows
   *                - Linux
   *                - ESX
   ********************************************************************************************
   * Created:     12.12.2018
   * Last Change: 13.12.2018
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   * - If new Import it works, if Database exist, it have Failures
   * - Windows
   *   - Domain and AD Values
   *   - Lot other Values
   * - Linux
   *   - Lot other Values
   ********************************************************************************************/
  SET @sSQL = '
  IF OBJECT_ID(''tempdb..' + @sTmpBIOS + ''') IS NOT NULL
    DROP TABLE #tmpBIOS'
  EXECUTE (@sSQL)

  SET @sSQL = '
  CREATE TABLE ' + @sTmpBIOS + ' (
    computerid              BIGINT,
    Manufacturer            NVARCHAR(256) COLLATE database_default,
    Caption                 NVARCHAR(256) COLLATE database_default,
    Version                 NVARCHAR(128) COLLATE database_default,
    SerialNumber            NVARCHAR(128) COLLATE database_default,
    ReleaseDate             DATETIME)'
  EXECUTE (@sSQL)

  SET @sSQL = '
  INSERT INTO ' + @sTmpBIOS + '
  SELECT
      bm.ComputerID,
      bm.Value AS Manufacturer,
      bc.Value AS Caption,
	  bv.Value AS Version,
      bs.Value AS SerialNumber,
      dbo.f_SQLDateTime(brd.Value) AS ReleaseDate
    FROM (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties(''CIM_BIOSElement'') WHERE Property = ''Manufacturer'') bm
      LEFT JOIN f_HardwareValuesByPropertyName(''Caption'') bc
	    ON bm.HardwareObjectID = bc.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''SerialNumber'') bs
	    ON bm.HardwareObjectID = bs.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''ReleaseDate'') brd
        ON bm.HardwareObjectID = brd.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''Version'') bv
        ON bm.HardwareObjectID = bv.HardwareObjectID'
  EXECUTE (@sSQL)

  SET @sSQL = '
  IF OBJECT_ID(''tempdb..' + @sTmpBaseBoard + ''') IS NOT NULL
    DROP TABLE ' + @sTmpBaseBoard
  EXECUTE (@sSQL)

  SET @sSQL = '
  CREATE TABLE ' + @sTmpBaseBoard + ' (
    computerid              BIGINT,
    Manufacturer            NVARCHAR(255) COLLATE database_default,
    Product                 NVARCHAR(255) COLLATE database_default,
    Version                 NVARCHAR(128) COLLATE database_default,
    SerialNumber            NVARCHAR(128) COLLATE database_default)'
  EXECUTE (@sSQL)

  SET @sSQL = '
  INSERT INTO ' + @sTmpBaseBoard + '
  SELECT
      bbm.ComputerID,
      bbm.Value AS Manufacturer,
      bbp.Value AS Product,
      bbv.Value AS Version,
      bbs.Value AS SerialNumber
    FROM (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties(''CIM_Card'') WHERE Property = ''Manufacturer'') bbm
      LEFT JOIN f_HardwareValuesByPropertyName(''Product'') bbp
        ON bbm.HardwareObjectID = bbp.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''SerialNumber'') bbs
	    ON bbm.HardwareObjectID = bbs.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''Version'') bbv
	    ON bbm.HardwareObjectID = bbv.HardwareObjectID'
  EXECUTE (@sSQL)

  SET @sSQL = '
  IF OBJECT_ID(''tempdb..' + @sTmpOS + ''') IS NOT NULL
    DROP TABLE ' + @sTmpOS
  EXECUTE (@sSQL)

  SET @sSQL = '
  CREATE TABLE ' + @sTmpOS + ' (
    computerid              BIGINT,
    [Language]              BIGINT,
    [SKU]                   INT,
    RegisteredUser          NVARCHAR(256) COLLATE database_default,
    SerialNumber            NVARCHAR(128) COLLATE database_default,
    ServicePackMajorVersion NVARCHAR(128) COLLATE database_default,
    ServicePackMinorVersion NVARCHAR(128) COLLATE database_default,
    LastBootUpTime          DATETIME)'
  EXECUTE (@sSQL)

  SET @sSQL = '
  INSERT INTO ' + @sTmpOS + '
  SELECT
      c.ComputerID,
      CAST(osLanguage.Value AS BIGINT)                      AS [Language],
      CAST(osSKU.Value AS INT)                              AS [SKU],
      osRegisteredUser.Value                                AS RegisteredUser,
      osSerial.Value                                        AS SerialNumber,
      osMajor.Value                                         AS ServicePackMajorVersion,
      osMinor.Value                                         AS ServePackMinorVersion,
      dbo.f_SQLDateTime(osLastBoot.Value)                   AS LastBootUpTime
    FROM ' + @sTmpFQDN + ' c
      LEFT JOIN f_HardwareProperties(''CIM_OperatingSystem'') osName
        ON c.ComputerID = osName.ComputerID
          AND osName.Property = ''Name'' AND osName.Value IS NOT NULL
      LEFT JOIN f_HardwareValuesByPropertyName(''OSLanguage'') osLanguage
        ON osName.HardwareObjectID = osLanguage.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''OperatingSystemSKU'') osSKU
        ON osName.HardwareObjectID = osSKU.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''SerialNumber'') osSerial
        ON osName.HardwareObjectID = osSerial.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''RegisteredUser'') osRegisteredUser
        ON osName.HardwareObjectID = osRegisteredUser.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''ServicePackMajorVersion'') osMajor
        ON osName.HardwareObjectID = osMajor.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''ServePackMinorVersion'') osMinor
        ON osName.HardwareObjectID = osMinor.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''LastBootUpTime'') osLastBoot
        ON osName.HardwareObjectID = osLastBoot.HardwareObjectID'
  EXECUTE (@sSQL)

  /* Alle mglichen Werte:
      (DataSourceID, SrcID, Site, Type, Name, DNSHostName, FQDN, Domain, PrimaryUserName, Description,
       LastBoot, LastScan, Manufacturer, Model, TotalMemoryMB, TotalDiskSpaceMB, SerialNumber, Virtual,
       WmiStatus, SamAccountName, SamAccountType, Enabled, PwdRequired, PwdCanChange, PwdExpires, NormalAccount,
       BiosCaption, BiosManufacturer, BiosReleaseDate, BiosSerialNumber, BiosSlicTable, BiosMsdmTable, BiosWindowsEdition,
       BiosVersion, BaseBoardManufacturer, BaseBoardProduct, BaseBoardSerialNumber, BaseBoardVersion, 
       OsArchitectureBits, OsFamily, OsInstallDate, OsLanguage, OsManufacturer, OsName, OsRegisteredUser,
       OsSerialNumber, OsServicePackMajorVersion, OsServicePackMinorVersion, OsSku, OsVersion,
       AdCn, AdCompany, AdCreated, AdDescription, AdDistinguishedName, AdLastLogon, AdLocation, AdLogonCount,
       AdModified, AdCountryCode, AdPwdLastSet, AdUserPrincipalName)
  */

  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblDevices]
    (DataSourceID, SrcID, Type, Name, DNSHostName, FQDN, Domain, PrimaryUserName, Description,
     LastBoot, LastScan, Manufacturer, Model, TotalMemoryMB, TotalDiskSpaceMB, SerialNumber, Virtual,
     BiosCaption, BiosManufacturer, BiosReleaseDate, BiosSerialNumber, BiosSlicTable, BiosMsdmTable, BiosWindowsEdition,
     BiosVersion, BaseBoardManufacturer, BaseBoardProduct, BaseBoardSerialNumber, BaseBoardVersion, 
     OsArchitectureBits, OsFamily, OsInstallDate, OsLanguage, OsManufacturer, OsName, OsRegisteredUser,
     OsSerialNumber, OsServicePackMajorVersion, OsServicePackMinorVersion, OsSku, OsVersion) 
  SELECT DISTINCT 
      ''' + @DataSourceID + ''' AS DataSourceID,
      COALESCE(dev.fqdn, CAST(c.ComputerID AS NVARCHAR(128))) AS SrcID,
      c.ChassisType AS Type,
      c.ComputerName AS Name,
      CAST(null AS NVARCHAR(256)) AS DNSHostName,
      dev.fqdn,
      CAST(null AS NVARCHAR(256)) AS Domain,
      c.LastLogin AS PrimaryUserName,
      CAST(null AS NVARCHAR(512)) AS Description,
      CAST(null AS DATETIME) AS LastBoot,
      c.InventoryDate AS LastScan,
      c.Manufacturer,
      c.Model,
      CAST(c.Memory*1024 AS BIGINT) AS TotalMemoryMB,
      CAST(c.DiskSize*1024 AS BIGINT) AS TotalDiskSpaceMB,
      c.SerialNumber,
      CASE
        WHEN c.SerialNumber LIKE ''VMware%'' THEN 1
        WHEN c.Manufacturer = ''Microsoft'' AND Model LIKE ''%HypeV%'' THEN 1
        ELSE 0
      END AS Virtual,
      b.Caption AS BiosCaption,
      b.Manufacturer AS BiosManufacturer,
      b.ReleaseDate  AS BiosReleaseDate,
      b.SerialNumber AS BiosSerialNumber,
      CAST(null AS INT) AS BiosSlicTable,
      CAST(null AS INT) AS BiosMsdmTable,
      CAST(null AS NVARCHAR(256)) AS BiosWindowsEdition,
      b.Version AS BiosVersion,
      bb.Manufacturer AS BaseBoardManufacturer,
      bb.Product AS BaseBoardProduct,
      bb.SerialNumber AS BaseBoardSerialNumber,
      bb.Version AS BaseBoardVersion,
      CAST(CASE
        WHEN os.Architecture LIKE ''%64%'' THEN 64
        WHEN os.Architecture LIKE ''%32%'' THEN 32
        WHEN os.Architecture LIKE ''x86'' THEN 32
        ELSE NULL
      END AS TINYINT) AS OsArchitectureBits,
      os.OSType AS OsFamily,
      os.InstallDate AS OsInstallDate,
      osE.Language AS OsLanguage,
      os.Manufacturer AS OsManufacturer,
      os.ProductName AS OsName,
      osE.RegisteredUser AS OsRegisteredUser,
      os.LicenseKey AS OsSerialNumber,
      osE.ServicePackMajorVersion AS OsServicePackMajorVersion,
      osE.ServicePackMinorVersion AS OsServicePackMinorVersion,
      osE.SKU AS OsSku,
      os.Version AS OsVersion
    FROM f_System_Base() c
      INNER JOIN ' + @sTmpFQDN + ' dev
        ON c.ComputerID = dev.computerid
      LEFT JOIN f_System_Chassis() ch
        ON c.ComputerID = ch.ComputerID
      LEFT JOIN f_System_OperatingSystem() os
        ON c.ComputerID = os.ComputerID
      LEFT JOIN ' + @sTmpOS + ' osE
        ON c.ComputerID = osE.computerid
      LEFT JOIN ' + @sTmpBIOS + ' b
        ON c.ComputerID = b.computerid
      LEFT JOIN ' + @sTmpBaseBoard + ' bb
        ON c.ComputerID = bb.computerid'
  EXECUTE (@sSQL)

  SET @sSQL = '
  DROP TABLE ' + @sTmpOS + '
  DROP TABLE ' + @sTmpBIOS + '
  DROP TABLE ' + @sTmpBaseBoard
  EXECUTE (@sSQL)

  /********************************************************************************************
   * DeviceLogons
   ********************************************************************************************
   * Description: Get Logon Information
   ********************************************************************************************
   * Created:     10.01.2019
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/
-- From Inventory
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblDeviceLogons]
    (DataSourceID, SrcDeviceID, UserDomain, UserName, LogonCount, TotalLogonMinutes)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                AS DataSourceID,
      usr.SrcDeviceID,
      usr.UserDomain                           AS UserDomain,
      usr.UserName,
      CAST(NULL AS INT) AS LogonCount,
      CAST(NULL AS INT) AS TotalLogonMinutes
    FROM (SELECT
              ir.ComputerID,
              COALESCE(dev.fqdn, CAST(ir.ComputerID AS NVARCHAR(128))) AS SrcDeviceID,
              u.UserCN                                                 AS UserName,
              IIF(o.IsDomain = 1, d.FlatName, dev.ComputerCN)          AS UserDomain
            FROM InventoryReport ir
              LEFT JOIN [User] u
                ON ir.UserID = u.UserID 
              LEFT JOIN [OrganizationFull] o
                ON u.UserOUID = o.OrganizationID
              LEFT JOIN [Domain] d
                ON o.DomainID = d.OrganizationID
              INNER JOIN ' + @sTmpFQDN + ' dev
                ON ir.ComputerID = dev.computerid
              WHERE u.UserCN IN (SELECT TOP 1 uTmp.UserCN
                                   FROM InventoryReport irTmp
                                     INNER JOIN [User] uTmp
                                       ON irTmp.UserID = uTmp.UserID
                                         AND uTmp.UserCN != ''SYSTEM''
                                   WHERE irTmp.ComputerID = ir.ComputerID
                                   ORDER By irTmp.SWDate DESC)) usr
    LEFT JOIN ' + @UniversalDB + '.[in].[tblDeviceLogons] l
      ON (usr.SrcDeviceID = l.SrcDeviceID)
       AND usr.UserDomain = l.UserDomain
       AND usr.UserName = l.UserName
    WHERE l.SrcDeviceID IS NULL'
	print @sSQL
  EXECUTE (@sSQL)

-- Last Login
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblDeviceLogons]
    (DataSourceID, SrcDeviceID, UserDomain, UserName, LogonCount, TotalLogonMinutes)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                    AS DataSourceID,
      usr.SrcDeviceID,
      COALESCE(usr.UserDomain, usr.ComputerCN)                     AS UserDomain,
      usr.UserName,
      CAST(NULL AS INT) AS LogonCount,
      CAST(NULL AS INT) AS TotalLogonMinutes
    FROM (SELECT
              u.ComputerID,
              dev.ComputerCN,
              COALESCE(dev.fqdn, CAST(u.ComputerID AS NVARCHAR(128))) AS SrcDeviceID,
              CASE
                WHEN CHARINDEX(''\'', u.Value) > 0 THEN LEFT(u.Value, CHARINDEX(''\'', u.Value) -1)
                WHEN CHARINDEX(''@'', u.Value) > 0 THEN RIGHT(u.Value, CHARINDEX(''@'', REVERSE(u.Value)) -1)
              END   AS UserDomain,
              CASE
                WHEN CHARINDEX(''\'', u.Value) > 0 THEN RIGHT(u.Value, CHARINDEX(''\'', REVERSE(u.Value)) -1)
                WHEN CHARINDEX(''@'', u.Value) > 0 THEN LEFT(u.Value, CHARINDEX(''@'', u.Value) -1)
                ELSE u.Value
              END   AS UserName	
            FROM f_HardwareProperties(''CIM_UnitaryComputerSystem'') u
              INNER JOIN ' + @sTmpFQDN + ' dev
                ON u.ComputerID = dev.computerid
            WHERE u.Property = ''UserName'') usr
      LEFT JOIN ' + @UniversalDB + '.[in].[tblDeviceLogons] l
        ON usr.SrcDeviceID = l.SrcDeviceID
          AND COALESCE(usr.UserDomain, usr.ComputerCN) = l.UserDomain
          AND usr.UserName = l.UserName
    WHERE l.SrcDeviceID IS NULL'
  EXECUTE (@sSQL)

  /********************************************************************************************
   * Users
   ********************************************************************************************
   * Description: Get Users
   ********************************************************************************************
   * Created:     10.01.2019
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/
-- From Inventory
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblUsers]
    (DataSourceID, SrcId, SrcDeviceID, Cn, UserPrincipalName, Domain, SamAccountName)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                          AS DataSourceID,
	  LEFT(COALESCE(usr.UserDomain + ''/'' + usr.UserName, CAST(usr.ComputerID AS NVARCHAR(16)) + usr.UserName) , 128)
                                                         AS SrcID,
      IIF(usr.UserDomain IS NULL, usr.SrcDeviceID, NULL) AS SrcDeviceID,
      usr.UserName                                       AS Cn,
      usr.UserPrincipalName                              AS UserPrincipalName,
      usr.UserDomain                                     AS Domain,
      usr.UserName                                       AS SamAccountName
    FROM (SELECT
              ir.ComputerID,
              COALESCE(dev.fqdn, CAST(ir.ComputerID AS NVARCHAR(128))) AS SrcDeviceID,
              u.UserCN                                        AS UserName,
              IIF(o.IsDomain = 1, d.FlatName, dev.ComputerCN) AS UserDomain,
              IIF(o.IsDomain = 1 AND CHARINDEX('','', o.DN) > 1, 
                    dev.ComputerCN + ''@'' + REPLACE(REPLACE(o.DN, ''DC='', ''''), '','', ''.''),
                    NULL)                                     AS UserPrincipalName
            FROM InventoryReport ir
              LEFT JOIN [User] u
                ON ir.UserID = u.UserID 
              LEFT JOIN [OrganizationFull] o
                ON u.UserOUID = o.OrganizationID
              LEFT JOIN [Domain] d
                ON o.DomainID = d.OrganizationID
              INNER JOIN ' + @sTmpFQDN + ' dev
                ON ir.ComputerID = dev.computerid
              WHERE u.UserCN IN (SELECT TOP 1 uTmp.UserCN
                                   FROM InventoryReport irTmp
                                     INNER JOIN [User] uTmp
                                       ON irTmp.UserID = uTmp.UserID
                                         AND uTmp.UserCN != ''SYSTEM''
                                   WHERE irTmp.ComputerID = ir.ComputerID
                                   ORDER By irTmp.SWDate DESC)) usr
    LEFT JOIN ' + @UniversalDB + '.[in].[tblUsers] l
      ON (usr.UserDomain IS NULL OR usr.SrcDeviceID = l.SrcDeviceID)
       AND usr.UserDomain = l.Domain
       AND usr.UserName = l.Cn
    WHERE l.SrcDeviceID IS NULL'
	print @sSQL
  EXECUTE (@sSQL)

-- Last Login
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblUsers]
    (DataSourceID, SrcId, SrcDeviceID, Cn, Domain, SamAccountName)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                    AS DataSourceID,
	  LEFT(COALESCE(usr.UserDomain + ''/'' + usr.UserName, CAST(usr.ComputerID AS NVARCHAR(16)) + usr.UserName) , 128)
                                                                   AS SrcID,
      IIF(usr.UserDomain IS NULL, usr.SrcDeviceID, NULL)           AS SrcDeviceID,
      usr.UserName                                                 AS Cn,
      COALESCE(usr.UserDomain, usr.ComputerCN)                     AS Domain,
      usr.UserName                                                 AS SamAccountName
    FROM (SELECT
              u.ComputerID,
              dev.ComputerCN,
              COALESCE(dev.fqdn, CAST(u.ComputerID AS NVARCHAR(128))) AS SrcDeviceID,
              CASE
                WHEN CHARINDEX(''\'', u.Value) > 0 THEN LEFT(u.Value, CHARINDEX(''\'', u.Value) -1)
                WHEN CHARINDEX(''@'', u.Value) > 0 THEN RIGHT(u.Value, CHARINDEX(''@'', REVERSE(u.Value)) -1)
              END   AS UserDomain,
              CASE
                WHEN CHARINDEX(''\'', u.Value) > 0 THEN RIGHT(u.Value, CHARINDEX(''\'', REVERSE(u.Value)) -1)
                WHEN CHARINDEX(''@'', u.Value) > 0 THEN LEFT(u.Value, CHARINDEX(''@'', u.Value) -1)
                ELSE u.Value
              END   AS UserName	
            FROM f_HardwareProperties(''CIM_UnitaryComputerSystem'') u
              INNER JOIN ' + @sTmpFQDN + ' dev
                ON u.ComputerID = dev.computerid
            WHERE u.Property = ''UserName'') usr
      LEFT JOIN ' + @UniversalDB + '.[in].[tblUsers] l
        ON (usr.UserDomain IS NULL OR usr.SrcDeviceID = l.SrcDeviceID)
          AND COALESCE(usr.UserDomain, usr.ComputerCN) = l.Domain
          AND usr.UserName = l.Cn
    WHERE l.SrcDeviceID IS NULL'
  EXECUTE (@sSQL)

  /********************************************************************************************
   * CPU's
   ********************************************************************************************
   * Description: Get all Processor
   ********************************************************************************************
   * Created:     12.12.2018
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblCpus]
    (DataSourceID, SrcDeviceID, DeviceID, DataWidth, Manufacturer, MaxClockSpeed, Name, NumberOfCores, NumberOfLogicalProcessors, HyperThreadingCapable)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                    AS DataSourceID,
	  COALESCE(dev.fqdn, CAST(p.ComputerID AS NVARCHAR(128)))      AS SrcDeviceID,
	  COALESCE(d.Value, CAST(p.HardwareObjectID AS NVARCHAR(256))) AS DeviceID,
	  CAST(dw.Value AS TINYINT)                                    AS DataWidth,
      CASE
        WHEN model.Value LIKE ''%Intel%'' THEN ''Intel''
        WHEN model.Value LIKE ''%AMD%''   THEN ''AMD''
        WHEN descr.Value LIKE ''%Intel%'' THEN ''Intel''
        WHEN descr.Value LIKE ''%AMD%''   THEN ''AMD''
        WHEN p.Value LIKE ''%PowerPC%''   THEN ''IBM''
        WHEN p.Value LIKE ''%SPARC%''     THEN ''SUN/Oracle''
        ELSE model.Value
      END                                                          AS Manufacturer,
	  CAST(sp.Value AS BIGINT)                                     AS MaxClockSpeed,
      COALESCE(p.Value, descr.Value)                               AS Name,
      CAST(core.Value AS INT)                                      AS NumberOfCores,
      CAST(logical.Value AS INT)                                   AS NumberOfLogicalProcessors,
      CASE
        WHEN model.Value LIKE ''%Intel%'' AND CAST(logical.Value AS INT) > CAST(core.Value AS INT) THEN 1
        WHEN model.Value LIKE ''%AMD%'' AND CAST(logical.Value AS INT) > CAST(core.Value AS INT) THEN 1
        WHEN descr.Value LIKE ''%Intel%'' AND CAST(logical.Value AS INT) > CAST(core.Value AS INT) THEN 1
        WHEN descr.Value LIKE ''%AMD%'' AND CAST(logical.Value AS INT) > CAST(core.Value AS INT) THEN 1
        ELSE 0
      END                                                          AS HyperThreadingCapable
    FROM (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties(''CIM_Processor'') WHERE Property = ''Name'') p
      INNER JOIN ' + @sTmpFQDN + ' dev
        ON p.ComputerID = dev.computerid
      LEFT JOIN f_HardwareValuesByPropertyName(''DeviceID'') d
        ON p.HardwareObjectID = d.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''DataWidth'') dw
        ON p.HardwareObjectID = dw.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''Manufacturer'') model
        ON p.HardwareObjectID = model.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''Description'') descr
        ON p.HardwareObjectID = descr.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''MaxClockSpeed'') sp
        ON p.HardwareObjectID = sp.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''NumberOfCores'') core
        ON p.HardwareObjectID = core.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''NumberOfLogicalProcessors'') logical
        ON p.HardwareObjectID = logical.HardwareObjectID'
EXECUTE (@sSQL)

  /********************************************************************************************
   * Memory
   ********************************************************************************************
   * Description: Get all Memory Information
   ********************************************************************************************
   * Created:     17.12.2018
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/

  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblMemory]
    (DataSourceID, SrcDeviceID, DeviceLocator, CapacityMB, DataWidth)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      COALESCE(MAX(dev.fqdn), CAST(m.ComputerID AS NVARCHAR(128))) AS SrcDeviceID,
      ''Bank: '' 
        + CAST(ROW_NUMBER()
                 OVER(PARTITION BY m.ComputerID, m.Property
                 ORDER BY m.ComputerID DESC) AS NVARCHAR(8))       AS DeviceLocator,
      CAST(Value AS BIGINT) / 1024 / 1024                          AS CapacityMB,
      CAST(NULL AS INT)                                            AS DataWidth
    FROM f_HardwareProperties(''Win32_PhysicalMemory'') m
      INNER JOIN ' + @sTmpFQDN + ' dev
        ON m.ComputerID = dev.computerid
    WHERE Property = ''Capacity''
    GROUP BY m.ComputerID, m.HardwareObjectID, m.Property, Value'
  EXECUTE (@sSQL)

  /********************************************************************************************
   * Physical Drives
   ********************************************************************************************
   * Description: Get all Physical Drive Informations
   ********************************************************************************************
   * Created:     31.12.2018
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/

  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblPhysicalDrives]
    (DataSourceID, SrcDeviceID, DeviceId, Name, Manufacturer, Model, Interface, SizeMB, SerialNumber)

  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      COALESCE(dev.fqdn, CAST(t.ComputerID AS NVARCHAR(128)))      AS SrcDeviceID,
      t.DeviceID,
      t.Name,
      t.Manufacturer,
      t.Model,
      t.Interface,
      t.SizeMB,
      t.SerialNumber
    FROM (SELECT 
              s.ComputerID,
              s.HardwareName                        AS DeviceID, 
              IIF(n.Value IS NULL, s.HardwareName, n.Value)
                                                    AS Name,
  --          ph.Value                              AS PhysicalIndex,
              ma.Value                              AS Manufacturer,
              mo.Value                              AS Model,
              it.Value                              AS Interface,
              CAST(s.Value AS BIGINT) / 1024 / 1024 AS SizeMB,
              sr.Value                              AS SerialNumber
            FROM f_HardwareProperties(''CIM_DiskDrive'') s
              LEFT JOIN f_HardwareValuesByPropertyName(''SerialNumber'') sr
                ON s.HardwareObjectID = sr.HardwareObjectID
              LEFT JOIN f_HardwareValuesByPropertyName(''Manufacturer'') ma
                ON s.HardwareObjectID = ma.HardwareObjectID
              LEFT JOIN f_HardwareValuesByPropertyName(''Model'') mo
                ON s.HardwareObjectID = mo.HardwareObjectID
  --          LEFT JOIN f_HardwareValuesByPropertyName(''PhysicalIndex'') ph
  --            ON s.HardwareObjectID = ph.HardwareObjectID
              LEFT JOIN f_HardwareValuesByPropertyName(''InterfaceType'') it
                ON s.HardwareObjectID = it.HardwareObjectID
              LEFT JOIN f_HardwareValuesByPropertyName(''Name'') n
                ON s.HardwareObjectID = n.HardwareObjectID
            WHERE s.Property = ''Size'' AND s.Class =''Win32_DiskDrive''
          UNION SELECT 
              s.ComputerID, 
              s.HardwareName                        AS DeviceID, 
  --          ASCII(SUBSTRING(s.HardwareName, 3, 1)) -97 AS PhysicalIndex,
              s.HardwareName                        AS Name,
              CAST(NULL AS NVARCHAR(256))           AS Manufacturer,
              CAST(NULL AS NVARCHAR(256))           AS Model,
              CAST(NULL AS NVARCHAR(256))           AS Interface,
              CAST(s.Value AS BIGINT) / 1024 / 1024 AS SizeMB,
              CAST(NULL AS NVARCHAR(256))           AS SerialNumber
            FROM f_HardwareProperties(''CIM_DiskDrive'') s
            WHERE s.Property = ''Size'' AND s.Class = ''MGS_DiskDrive''
              AND s.HardwareName LIKE ''%[a-z]''
          UNION SELECT 
              s.ComputerID, 
              s.HardwareName                        AS DeviceID, 
  --          CAST(NULL AS NVARCHAR(256))           AS PhysicalIndex,
              s.HardwareName                        AS Name,
              CAST(NULL AS NVARCHAR(256))           AS Manufacturer,
              CAST(NULL AS NVARCHAR(256))           AS Model,
              CAST(NULL AS NVARCHAR(256))           AS Interface,
              CAST(s.Value AS BIGINT) / 1024 / 1024 AS SizeMB,
              CAST(NULL AS NVARCHAR(256))           AS SerialNumber
            FROM f_HardwareProperties(''CIM_DiskDrive'') s
            WHERE s.Property = ''Size'' AND s.Class NOT IN (''Win32_DiskDrive'', ''MGS_DiskDrive'')) t
      INNER JOIN ' + @sTmpFQDN + ' dev
        ON t.ComputerID = dev.computerid'
  EXECUTE (@sSQL)

  /********************************************************************************************
   * Logical Drives
   ********************************************************************************************
   * Description: Get all Logical Drive Informations
   ********************************************************************************************
   * Created:     31.12.2018
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/

  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblLogicalDrives]
    (DataSourceID, SrcDeviceID, SrcPhysicalDriveId, VolumeName, DriveLetter, Description, Type, FileSystem, SizeMB, FreeSpaceMB, Wwn)

  SELECT DISTINCT
      ''' + @DataSourceID + '''  AS DataSourceID,
      COALESCE(dev.fqdn, CAST(t.ComputerID AS NVARCHAR(128))) AS SrcDeviceID,
      t.SrcPhysicalDriveID,
      t.VolumeName,
      t.DriveLetter,
      t.Description,
      t.Type,
      t.FileSystem,
      t.SizeMB,
      t.FreeSpaceMB,
      t.Wwn
    FROM (SELECT 
              ld.ComputerID, 
              d.HardwareName    AS SrcPhysicalDriveID,
              ld.VolumeName,
              ld.HardwareName   AS DriveLetter, 
              ld.Description,
              CAST(ld.MediaType AS INT) AS Type,
              ld.FileSystem,
  --          d.[Index]         AS PhysicalIndex,
              CAST(ld.Size AS BIGINT) / 1024 / 1204  AS SizeMB, 
              CAST(ld.FreeSpace AS BIGINT) / 1024 /1024 AS FreeSpaceMB, 
              CAST(null AS NVARCHAR(256))  AS Wwn
  --          ld.VolumeSerialNumber
            FROM LogicalDisk ld 
              LEFT JOIN (SELECT
                             dda.ComputerID,
                             REPLACE(RIGHT(LEFT(dda.Value, LEN(dda.Value)-1), CHARINDEX(''""'', REVERSE(dda.Value), 2)-2), ''\\'', ''\'') AS Antecedent,
                             RIGHT(LEFT(ldd.Value, LEN(ldd.Value)-1), CHARINDEX(''""'', REVERSE(ldd.Value), 2)-2) AS Dependent
                           FROM f_HardwareProperties(''Win32_DiskDriveToDiskPartition'') dda
                             LEFT JOIN f_HardwareValuesByPropertyName(''Dependent'') ddd
                               ON dda.HardwareObjectID = ddd.HardwareObjectID
                             INNER JOIN f_HardwareProperties(''Win32_LogicalDiskToPartition'') lda
                               ON lda.Property = ''Antecedent''
                                 AND dda.ComputerID = lda.ComputerID
                                 AND ddd.Value = lda.Value
                             LEFT JOIN f_HardwareValuesByPropertyName(''Dependent'') ldd
                               ON lda.HardwareObjectID = ldd.HardwareObjectID
                           WHERE dda.Property = ''Antecedent'') lDisk
                ON ld.ComputerID = lDisk.ComputerID
                  AND ld.HardwareName = lDisk.Dependent
              LEFT JOIN DiskDrive d
                ON ld.ComputerID = d.ComputerID
                  AND lDisk.Antecedent = d.HardwareName
            WHERE ld.DriveType = 3
          UNION SELECT 
              s.ComputerID, 
              LEFT(s.HardwareName, PATINDEX(''%[a-z][0-9]%'', s.HardwareName)) AS SrcPhysicalDriveID,
              s.HardwareName                              AS VolumeName,
              s.HardwareName                              AS DriveLetter,
              CAST(NULL AS NVARCHAR(256))                 AS [Description],
              CAST(NULL AS INT)                           AS Type,
              CAST(NULL AS NVARCHAR(256))                 AS FileSystem,
  --          ASCII(SUBSTRING(s.HardwareName, 3, 1)) -97  AS PhysicalIndex,
              CAST(s.Value AS BIGINT) / 1024 / 1204       AS SizeMB, 
              CAST(NULL AS INT) / 1024 /1024              AS FreeSpaceMB, 
              CAST(NULL AS NVARCHAR(256))                 AS Wwn
  --          NULL                                        AS VolumeSerialNumber 
            FROM f_HardwareProperties(''CIM_DiskDrive'') s
            WHERE s.Property = ''Size'' AND s.Class = ''MGS_DiskDrive''
              AND s.HardwareName LIKE ''%[a-z][0-9]%'') t
      INNER JOIN ' + @sTmpFQDN + ' dev
        ON t.ComputerID = dev.computerid
    WHERE t.SrcPhysicalDriveID IS NOT NULL'
  EXECUTE (@sSQL)

  /********************************************************************************************
   * Services
   ********************************************************************************************
   * Description: Get all Logical Drive Informations
   ********************************************************************************************
   * Created:     31.12.2018
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblServices]
    (DataSourceID, SrcDeviceID, Name, DisplayName, Description, InstallLocation, Account, Started, StartMode, Status)

  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      COALESCE(dev.fqdn, CAST(sn.ComputerID AS NVARCHAR(128)))     AS SrcDeviceID,
      sn.Value                                                     AS Name,
      sdn.Value                                                    AS DisplayName,
      sd.Value                                                     AS Description,
      spn.Value                                                    AS InstallLocation,
      ssn.Value                                                    AS Account,
      IIF(ss.Value IN (''Running''), 1, 0)                           AS Started,
      ssm.Value                                                    AS StartMode,
      sst.Value                                                    AS Status
    FROM f_HardwareProperties(''Win32_Service'') sn
      INNER JOIN ' + @sTmpFQDN + ' dev
        ON sn.ComputerID = dev.computerid
      LEFT JOIN f_HardwareValuesByPropertyName(''DisplayName'') sdn
        ON sn.HardwareObjectID = sdn.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''Description'') sd
        ON sn.HardwareObjectID = sd.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''State'') ss
        ON sn.HardwareObjectID = ss.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''StartMode'') ssm
        ON sn.HardwareObjectID = ssm.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''Status'') sst
        ON sn.HardwareObjectID = sst.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''StartName'') ssn
        ON sn.HardwareObjectID = ssn.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''PathName'') spn
        ON sn.HardwareObjectID = spn.HardwareObjectID
    WHERE sn.Property = ''Name'''
  EXECUTE (@sSQL)

  /********************************************************************************************
   * Software
   ********************************************************************************************
   * Description: Get all Software
   *                - File
   *                - ARP
   *                - MSI
   *                - ISO Tag
   *                - Sharepoint Edition
   ********************************************************************************************
   * Created:     13.12.218
   * Last Change:
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   * - Oracle
   ********************************************************************************************/
	  
  -- ARP and RA
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblSoftware] 
    (DataSourceID, SrcDeviceID, Publisher, DiscoveredProduct, 
     DiscoveredVersion, FileName, InstallLocation, InstallDate, 
     SoftwareCode, SwidUniqueId, SwidLicensorId)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      COALESCE(d.fqdn, CAST(d.ComputerID AS NVARCHAR(128))) AS SrcDeviceID,
      SUBSTRING(RTRIM(LTRIM(REPLACE(ISNULL(sd.Publisher, ''''), ''%'', ''_''))), 1, 256) AS Publisher,
      SUBSTRING(RTRIM(LTRIM(REPLACE(ISNULL(CASE
                                             WHEN ISNULL(sd.RawSoftwareName, '''') = '''' THEN sv.SoftwareName
                                             ELSE sd.RawSoftwareName
                                           END, ''''), ''%'', ''_''))), 1, 256)          AS DiscoveredProduct,
      SUBSTRING(RTRIM(LTRIM(REPLACE(ISNULL(CASE
                                             WHEN ISNULL(sd.RawVersion, '''') = '''' THEN sv.Version
                                             ELSE sd.RawVersion
                                           END, ''''), ''%'', ''_''))), 1, 256)           AS DiscoveredVersion,
      CAST(null AS NVARCHAR(256)) AS FileName,
      CAST(null AS NVARCHAR(1024)) AS InstallLocation,
      so.InstallationDate                                                           AS InstallDate,
      val.Value AS SoftwareCode,
      CAST(null AS NVARCHAR(256)) AS SwidUniqueId,
      CAST(null AS NVARCHAR(256)) AS SwidLicensorId
    FROM dbo.SoftwareOccurrence AS so
      INNER JOIN ' + @sTmpFQDN + ' d
        ON so.ComputerID = d.ComputerID
      INNER JOIN dbo.SoftwareVersion AS sv 
        ON so.SoftwareID = sv.SoftwareID
      LEFT JOIN dbo.SoftwareDetails AS sd
        ON so.SoftwareDetailsID = sd.SoftwareDetailsID
      LEFT JOIN (SELECT tmpSv.SoftwareOccurrenceID, tmpSp.Property, tmpSv.Value
                   FROM dbo.SoftwareValue tmpSv
                     LEFT JOIN dbo.SoftwareProperty tmpSp
                       ON tmpSv.SoftwarePropertyID = tmpSp.SoftwarePropertyID
                   WHERE tmpSp.Property in (''DisplayName'')
                ) dpn
        ON so.SoftwareOccurrenceID = dpn.SoftwareOccurrenceID
      LEFT JOIN dbo.SoftwareValue AS val
        ON so.SoftwareOccurrenceID = val.SoftwareOccurrenceID
          AND val.SoftwarePropertyID IN (SELECT
                                             SoftwarePropertyID
                                           FROM dbo.SoftwareProperty
                                           WHERE Property = ''ProductCode''
                                        )
    WHERE RTRIM(LTRIM(sv.SoftwareName)) <> ''''
      AND (so.Evidence IN (''Uninstall'',''MSI'',''ManageSoft + MSI'')
        OR (so.Evidence = ''ManageSoft'' AND dpn.Value = sd.RawSoftwareName))'
  EXECUTE (@sSQL)

  -- Filescan and ISO Tac
  SET @sSQL = '
  WITH IsoTAG (SoftwareIsoTagFileID, Publisher, ProductName, ProductVersion, 
               SoftwareUniqueID, SoftwareTagID, LicensorID)
    AS (SELECT
            itf.SoftwareIsoTagFileID,
            itfx.x.value(''(/*:SoftwareIdentity/*:Entity/@name)[1]'', 
                         ''nvarchar(256)'')                       AS Publisher,
            itfx.x.value(''(/*:SoftwareIdentity/@name)[1]'', 
                         ''nvarchar(256)'')                       AS ProductName,
            itfx.x.value(''(/*:SoftwareIdentity/@version)[1]'', 
                         ''nvarchar(256)'')                       AS ProductVersion,
            itfx.x.value(''(/*:SoftwareIdentity/@uniqueId)[1]'', 
                         ''nvarchar(256)'')                       AS SoftwareUniqueID,
            itfx.x.value(''(/*:SoftwareIdentity/@tagId)[1]'', 
                         ''nvarchar(256)'')                       AS SoftwareTagID,
            itfx.x.value(''(/*:SoftwareIdentity/*:Entity/@regid)[1]'', 
                         ''nvarchar(256)'')                       AS LicensorID
          FROM SoftwareIsoTagFile itf WITH (NOLOCK)
            CROSS APPLY (SELECT CAST(CAST(
                                       REPLACE(itf.TagContent, 
                                       ''UTF-8'', ''ISO-8859-1'') AS VARCHAR(MAX)) AS XML) x) itfx
          WHERE itf.TagContent LIKE ''%SoftwareIdentity%'')

  INSERT INTO ' + @UniversalDB + '.[in].[tblSoftware] 
    (DataSourceID, SrcDeviceID, Publisher, DiscoveredProduct, 
     DiscoveredVersion, FileName, InstallLocation, InstallDate, 
     SoftwareCode, SwidUniqueId, SwidLicensorId)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
	  COALESCE(d.fqdn, CAST(d.ComputerID AS NVARCHAR(128))) AS SrcDeviceID, 
      COALESCE(ite.Name, IsoTAG.Publisher, 
               itf.OriginalArpPublisher, sf.CompanyName)    AS Publisher,
      COALESCE(itsv.ProductTitle, IsoTAG.ProductName, itf.OriginalArpDisplayName,
               sf.FileDescription)                          AS DiscoveredProduct,
      COALESCE(itsv.ProductVersionName, IsoTAG.ProductVersion,  itf.OriginalArpDisplayVersion,
                sf.FileVersion)                             AS DiscoveredVersion,
      LEFT(sfn.Name, 256)                                   AS FileName,
      LEFT(sfp.Path, 1024)                                  AS InstallLocation,
      sf.Datetime                                           AS InstallDate,
      itf.OriginalArpGuid                                   AS SoftwareCode,
      COALESCE(itsv.SoftwareUniqueID, IsoTAG.SoftwareUniqueID, 
	           IsoTAG.SoftwareTagID)                        AS SwidUniqueId,
      COALESCE(ite.RegID, IsoTAG.LicensorID)                AS SwidLicensorId
    FROM SoftwareFile sf WITH (NOLOCK)
      INNER JOIN ' + @sTmpFQDN + ' d
        ON sf.ComputerID = d.ComputerID
      LEFT JOIN SoftwareIsoTagFile itf WITH (NOLOCK)
        ON sf.SoftwareIsoTagFileID = itf.SoftwareIsoTagFileID
      LEFT JOIN dbo.SoftwareFilePath sfp WITH (NOLOCK)
        ON sf.SoftwareFilePathID = sfp.SoftwareFilePathID
      LEFT JOIN dbo.SoftwareFileName sfn WITH (NOLOCK)
        ON sf.SoftwareFileNameID = sfn.SoftwareFileNameID
      LEFT JOIN SoftwareIsoTagSoftwareVersion itsv WITH (NOLOCK)
        ON itf.SoftwareIsoTagSoftwareVersionID = itsv.SoftwareIsoTagSoftwareVersionID
      LEFT JOIN SoftwareIsoTagEntity ite WITH (NOLOCK)
        ON itf.SoftwareCreatorEntityID = ite.SoftwareIsoTagEntityID
      LEFT JOIN SoftwareVersion sv WITH (NOLOCK)
        ON sf.SoftwareID = sv.SoftwareID
      LEFT JOIN IsoTAG
        ON itf.SoftwareIsoTagFileID = IsoTAG.SoftwareIsoTagFileID
    WHERE itf.SoftwareIsoTagFileID > 0 OR LEN(sf.FileDescription) > 0'
  EXECUTE (@sSQL)

  -- SharePoint
  -- Open points:
  -- - InstallDate
  -- - InstallLocation
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblSoftware] 
    (DataSourceID, SrcDeviceID, Publisher, DiscoveredProduct, 
     DiscoveredVersion, FileName, InstallLocation, InstallDate, 
     SoftwareCode, SwidUniqueId, SwidLicensorId)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      COALESCE(d.fqdn, CAST(d.ComputerID AS NVARCHAR(128))) AS SrcDeviceID,
      ''Microsoft''                                           AS Publisher,
      LEFT(s.FullProductName, 256)                          AS DiscoveredProduct,
      LEFT(s.Version, 256)                                  AS DiscoveredVersion,
      CAST(NULL AS NVARCHAR(256))                           AS FileName,
      CAST(NULL AS NVARCHAR(1024))                          AS InstallLocation,
      CAST(NULL AS DATETIME)                                AS InstallDate,
      s.SKUID                                               AS SoftwareCode,
      CAST(null AS NVARCHAR(256))                           AS SwidUniqueId,
      CAST(null AS NVARCHAR(256))                           AS SwidLicensorId
    FROM f_Microsoft_SharePoint() s
      INNER JOIN ' + @sTmpFQDN + ' d
        ON s.ComputerID = d.ComputerID'
 EXECUTE (@sSQL)

  -- OS
  /*
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblSoftware] 
    (DataSourceID, SrcDeviceID, Publisher, DiscoveredProduct, 
     DiscoveredVersion, FileName, InstallLocation, InstallDate, 
     SoftwareCode, SwidUniqueId, SwidLicensorId)
  SELECT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      COALESCE(d.fqdn, CAST(d.ComputerID AS NVARCHAR(128))) AS SrcDeviceID,
      LEFT(o.Manufacturer, 256)                             AS Publisher,
      LEFT(o.ProductName, 256)                              AS DiscoveredProduct,
      LEFT(o.Version, 256)                                  AS DiscoveredVersion,
      CAST(NULL AS NVARCHAR(256))                           AS FileName,
      CAST(NULL AS NVARCHAR(1024))                          AS InstallLocation,
      o.InstallDate                                         AS InstallDate,
      CAST(null AS NVARCHAR(256))                           AS SoftwareCode,
      CAST(null AS NVARCHAR(256))                           AS SwidUniqueId,
      CAST(null AS NVARCHAR(256))                           AS SwidLicensorId
    FROM f_System_OperatingSystem() o
      INNER JOIN ' + @sTmpFQDN + ' d
        ON o.ComputerID = d.ComputerID
  EXECUTE (@sSQL)
  */


  /********************************************************************************************
   * Virtualisation
   ********************************************************************************************
   * Description: Get all Virtualisationinformation from:
   *                - VMWare
   *                - HyperV
   ********************************************************************************************
   * Created:     14.12.218
   * Last Change:
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   * - HyperV 
   *   - Powerstate
   *   - MigrationEnabled
   *   - AffinityRuleHostId
   *   - MemoryReservation,
   *   - CpuReservation,
   *   - CpuAssigned
   * - VMWARE
   *   - HaModeEnabled, 
   *   - DrsModeEnabled
   *   - MigrationEnabled
   *   - AffinityRuleHostId
   ********************************************************************************************/

  -- VMWare u. HyperV

  -- [UI_test3].[in].[tblVmFarms]
  -- VMWare ClusterName
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblVmFarms] (DataSourceId, Name, HaModeEnabled, DrsModeEnabled)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      esxCl.Value          AS Name,
      CAST(NULL AS BIT)    AS HaModeEnabled,
      CAST(NULL AS BIT)    AS DrsModeEnabled
    FROM ' + @sTmpFQDN + ' esx
      INNER JOIN (SELECT ComputerID, HardwareName, Value FROM f_HardwareProperties(''MGS_Cluster'') WHERE Property = ''ElementName'') esxCl
        ON esx.ComputerID = esxCl.ComputerID'
  EXECUTE (@sSQL)

  -- HyperV ClusterName
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[in].[tblVmFarms] (DataSourceId, Name, HaModeEnabled, DrsModeEnabled)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      cl.ClusterName       AS Name,
      cl.IsFailover        AS HaModeEnabled,
      CAST(NULL AS BIT)    AS DrsModeEnabled
    FROM f_Virtual_HyperV_Cluster() cl
      INNER JOIN ' + @sTmpFQDN + ' c
        ON cl.ComputerID = c.ComputerID'
  EXECUTE (@sSQL)

  -- [UI_test3].[IN].[tblVmHosts]
  -- VMWare
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[IN].[tblVmHosts] (DataSourceId, SrcDeviceId, SrcId, FarmName, Name, MigrationEnabled, OsVersion, NumberOfCores)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      f.fqdn               AS SrcDeviceId,         -- Linket to
      f.fqdn               AS SrcId,               -- srcVmDeviceID to witch the Clients will be linked
      vCluster.Value       AS FarmName,            -- Link to VmFarms.Name
      f.computercn         AS Name,
      CAST(NULL AS BIT)    AS MigrationEnabled,
      osVersion.Value      AS OsVersion,
      cores.NumberOfCores  AS NumberOfCores
    FROM (SELECT DISTINCT ComputerID, HardwareObjectID, Value 
            FROM f_HardwareProperties(''VMWARE_VMComputerSystem'') 
            WHERE Property IN (''Hostname'', ''Elementname'')) esx
      INNER JOIN ' + @sTmpFQDN + ' f
        ON esx.ComputerID = f.computerid
      LEFT JOIN (SELECT ComputerID, HardwareName, Value FROM f_HardwareProperties(''MGS_Cluster'') WHERE Property = ''ElementName'') vCluster
        ON esx.ComputerId = vCluster.ComputerID
      LEFT JOIN f_HardwareProperties(''CIM_OperatingSystem'') osVersion
        ON esx.ComputerID = osVersion.ComputerID
          AND osVersion.Property = ''Version''
      LEFT JOIN (SELECT p.ComputerID, SUM(p.Cores) AS NumberOfCores FROM f_System_Processor() p GROUP BY p.ComputerID) cores
        ON esx.ComputerID = cores.ComputerID'
  EXECUTE (@sSQL)

  -- HyperV
  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[IN].[tblVmHosts] (DataSourceId, SrcDeviceId, SrcId, FarmName, Name, MigrationEnabled, OsVersion, NumberOfCores)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      f.fqdn               AS SrcDeviceId,         -- Linket to
	  f.fqdn               AS SrcId,               -- srcVmDeviceID to witch the Clients will be linked
      Cluster.ClusterName  AS FarmName,            -- Link to VmFarms.Name
      f.computercn         AS Name,
      CAST(NULL AS BIT)    AS MigrationEnabled,
      osVersion.Value      AS OsVersion,
      cores.NumberOfCores  AS NumberOfCores
    FROM (SELECT DISTINCT ComputerID FROM f_Virtual_HyperV_HostInfo()) HyperV
      INNER JOIN ' + @sTmpFQDN + ' f
        ON HyperV.ComputerID = f.computerid
      LEFT JOIN f_Virtual_HyperV_Cluster() Cluster
        ON HyperV.ComputerID = Cluster.ComputerID
      LEFT JOIN f_HardwareProperties(''CIM_OperatingSystem'') osVersion
        ON HyperV.ComputerID = osVersion.ComputerID
          AND osVersion.Property = ''Version''
      LEFT JOIN (SELECT p.ComputerID, SUM(p.Cores) AS NumberOfCores FROM f_System_Processor() p GROUP BY p.ComputerID) cores
        ON HyperV.ComputerID = cores.ComputerID'
  EXECUTE (@sSQL)

  -- [UI_test3].[IN].[tblVms]
  SET @sSQL = '
  IF OBJECT_ID(''tempdb..' + @sTmpTblVms + ''') IS NOT NULL
    DROP TABLE ' + @sTmpTblVms
  EXECUTE (@sSQL)

  SET @sSQL = '
  CREATE TABLE ' + @sTmpTblVms + ' (
    DataSourceId       VARCHAR(32)   COLLATE database_default,
    SrcDeviceId        NVARCHAR(128) COLLATE database_default,
    SrcVmHostId        NVARCHAR(128) COLLATE database_default,
    AffinityRuleHostId NVARCHAR(128) COLLATE database_default,
    PowerState         VARCHAR(16)   COLLATE database_default,
    MemoryReservation  INT, 
    CpuReservation     INT,
    CpuAssigned        INT,
    Name               NVARCHAR(256) COLLATE database_default, 
    DNSName            NVARCHAR(256) COLLATE database_default,
    IsSnapShot         BIT, 
    IsTemplate         BIT, 
    GuestHealth        NVARCHAR(256) COLLATE database_default,
    HostInventoryDate  DATETIME)'
  EXECUTE (@sSQL)

  -- VMWare
  SET @sSQL = '
 INSERT INTO ' + @sTmpTblVms + '
 SELECT DISTINCT
   ''' + @DataSourceID + '''                                           AS DataSourceID,
   c.fqdn                      AS SrcDeviceId,         -- Linket to Device (Client)
   esx.ESXFQDN                 AS SrcVmHostId,         -- Linked to VMHosts (ESX)
   CAST(NULL AS NVARCHAR(256)) AS AffinityRuleHostId,  -- Feste bindung an einen Server
   CASE
    WHEN esx.PowerState = ''poweredOn''  THEN ''On''
    WHEN esx.PowerState = ''poweredOff'' THEN ''Off''
    ELSE esx.PowerState
   END                         AS PowerState,
   esx.MemoryReservation       AS MemoryReservation,
   esx.CpuReservation          AS CpuReservation,
   esx.numCpu                  AS CpuAssigned,
   c.ComputerCN                AS Name,
   c.fqdn                      AS DNSName,
   CAST(NULL AS BIT)           AS IsSnapShot,
   CAST(NULL AS BIT)           AS IsTemplate,
   CAST(''OK'' AS NVARCHAR(256)) AS GuestHealth,
   ir.HWDate                   AS HostInventoryDate
  FROM ' + @sTmpFQDN + ' c
   INNER JOIN 
		 (SELECT DISTINCT
			esx.ComputerID          AS ESXComputerID,
			esx.ComputerCN          AS ESXComputerCN,
			esx.fqdn                AS ESXFQDN,
			link.GuestID            AS GuestComputerID,
			pw.Value                AS PowerState,
			CAST(mem.Value  AS INT) AS MemoryReservation, 
			CAST(cpu.Value  AS INT) AS numCpu,
			CAST(cpur.Value AS INT) AS CpuReservation
		   FROM ' + @sTmpFQDN + ' esx WITH (NOLOCK)
			LEFT JOIN
				 (SELECT
					CIM.ComputerID           GuestID, 
					VMWARE.ComputerID        HostID,
					VMWARE.HardwareObjectID  VMWareGuestHWID
				   FROM f_HardwareProperties(''CIM_BIOSElement'') AS CIM
					LEFT JOIN (SELECT ComputerID, HardwareObjectID, REPLACE(REPLACE(Value,'' '',''''),''-'','''') Value
								FROM f_HardwareProperties(''VMWARE_VMComputerSystem'') 
								WHERE Property IN (''UUID'')
							  ) AS VMWARE
					 ON REPLACE(REPLACE(REPLACE(LEFT(CIM.Value,100),'' '',''''),''-'',''''),''VMware'','''') = VMWARE.Value
				   WHERE CIM.Property = ''SerialNumber'' AND VMWARE.ComputerID IS NOT NULL
					AND CIM.ComputerID <> VMWARE.ComputerID 
				  UNION SELECT
					tmpC.ComputerID           GuestID,
					VMHName.ComputerID        HostID,
					VMHName.HardwareObjectID  VMWareGuestHWID
				   FROM f_HardwareProperties(''VMWARE_VMComputerSystem'') VMHName
					LEFT JOIN ' + @sTmpFQDN + ' AS tmpC
					 ON VMHName.Value = tmpC.FQDN
				   WHERE VMHName.Property = ''HostName'' AND tmpC.ComputerID <> VMHName.ComputerID
				  UNION SELECT
					tmpC.ComputerID           GuestID,
					VMHName.ComputerID        HostID,
					VMHName.HardwareObjectID  VMWareGuestHWID
				   FROM f_HardwareProperties(''VMWARE_VMComputerSystem'') VMHName
					LEFT JOIN ' + @sTmpFQDN + ' tmpC WITH (NOLOCK)
					 ON VMHName.Value = tmpC.ComputerCN
					LEFT JOIN f_HardwareValuesByPropertyName(''IPAddress'') vmIP
					 ON vmIP.HardwareObjectID = VMHName.HardwareObjectID
					INNER JOIN f_HardwareProperties(''CIM_MGS_NetworkAdapterConfiguration'') nicIP
					 ON tmpc.ComputerID = nicIP.ComputerID
					  AND nicIP.Property = ''IPAddress''
					  AND vmIP.Value = nicIP.Value
				   WHERE VMHName.Property = ''Elementname'' AND tmpC.ComputerID <> VMHName.ComputerID
				 ) AS link
			 ON esx.ComputerID = link.HostID
			LEFT JOIN f_HardwareValuesByPropertyName(''PowerState'') pw
			 ON link.VMWareGuestHWID = pw.HardwareObjectID
			LEFT JOIN f_HardwareValuesByPropertyName(''memoryReservation'') mem
			 ON link.VMWareGuestHWID = mem.HardwareObjectID
			LEFT JOIN f_HardwareValuesByPropertyName(''numCpu'') cpu
			 ON link.VMWareGuestHWID = cpu.HardwareObjectID
			LEFT JOIN f_HardwareValuesByPropertyName(''CpuReservation'') cpur
			 ON link.VMWareGuestHWID = cpur.HardwareObjectID
		   WHERE link.GuestID > 0
		 ) esx
	ON c.ComputerID = esx.GuestComputerID
   LEFT JOIN InventoryReport ir
	ON esx.ESXComputerID = ir.ComputerID'
  EXECUTE (@sSQL)
		
  -- HyperV
  SET @sSQL = '
  INSERT INTO ' + @sTmpTblVms + '
  SELECT DISTINCT
      ''' + @DataSourceID + '''   AS DataSourceID,
      gd.fqdn                     AS SrcDeviceId,         -- Linket to Device (Client)
      hd.fqdn                     AS SrcVmHostId,         -- Linked to VMHosts (ESX)
      CAST(NULL AS NVARCHAR(256)) AS AffinityRuleHostId,  -- Feste bindung an einen Server
      CAST(''On'' AS VARCHAR(16))   AS PowerState,
      CAST(NULL AS INT)           AS MemoryReservation,
      CAST(NULL AS INT)           AS CpuReservation,
      CAST(NULL AS INT)           AS CpuAssigned,
      gd.computercn               AS Name,
      gd.fqdn                     AS DNSName,
      CAST(NULL AS BIT)           AS IsSnapShot,
      CAST(NULL AS BIT)           AS IsTemplate,
      CAST(''OK'' AS NVARCHAR(256)) AS GuestHealth,
      ir.HWDate                   AS HostInventoryDate
    FROM f_Virtual_HyperV_GuestRelation(' + IIF(@HyperVWithSerial=1, '1', '0')  + ') g
      LEFT JOIN ' + @sTmpFQDN + ' gd
        ON g.GuestID = gd.computerid
      LEFT JOIN ' + @sTmpFQDN + ' hd
        ON g.HostID = hd.computerid
      LEFT JOIN f_Virtual_HyperV_Cluster() cl
        ON g.HostID = cl.ComputerID
      LEFT JOIN InventoryReport ir
        ON g.HostID = ir.ComputerID'
  EXECUTE (@sSQL)

  SET @sSQL = '
  DELETE v
  FROM ' + @sTmpTblVms + ' v
    INNER JOIN (SELECT t.srcDeviceID, MIN(t.HostInventoryDate) AS HostInventoryDate
                  FROM ' + @sTmpTblVms + ' t
                  GROUP BY t.SrcDeviceID, t.PowerState
                  HAVING COUNT(HostInventoryDate) > 1) x
      ON v.SrcDeviceId = x.SrcDeviceId
        AND v.HostInventoryDate = x.HostInventoryDate'
  EXECUTE (@sSQL)

  SET @sSQL = '
  DELETE v
  FROM ' + @sTmpTblVms + ' v
    INNER JOIN (SELECT t.srcDeviceID
                  FROM ' + @sTmpTblVms + ' t
                  GROUP BY t.SrcDeviceID
                  HAVING COUNT(SrcDeviceId) > 1) x
      ON v.SrcDeviceId = x.SrcDeviceId
        AND v.PowerState != ''On'''
  EXECUTE (@sSQL)

  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[IN].[tblVms] (DataSourceId, SrcDeviceId, SrcVmHostId, AffinityRuleHostId,
                                      PowerState, MemoryReservation, CpuReservation, CpuAssigned,
                                      Name, DNSName, IsSnapShot, IsTemplate, GuestHealth)
  SELECT DISTINCT
      DataSourceId, 
      SrcDeviceId, 
      SrcVmHostId, 
      AffinityRuleHostId,
      PowerState, 
      MemoryReservation, 
      CpuReservation, 
      CpuAssigned,
      Name, 
      DNSName, 
      IsSnapShot, 
      IsTemplate, 
      GuestHealth
    FROM ' + @sTmpTblVms
  EXECUTE (@sSQL)

  SET @sSQL = '
  DROP TABLE ' + @sTmpTblVms
  EXECUTE (@sSQL)

  /********************************************************************************************
   * Microsoft SQL
   ********************************************************************************************
   * Description: Get all SQL Server Information
   ********************************************************************************************
   * Created:     14.12.218
   * Last Change:
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   * - IsIntegratedSecurity
   * - StartMode  (Service Start Mode)
   * - Enviroment (User supplied info to allow license assignment. E.g. production vs. dev/test)
   * - Isv        (Specify vendor if included in separate ISV license.)
   ********************************************************************************************/

  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[IN].[tblDatabaseServers] (DataSourceID, SrcDeviceID, SrcID, InstallLocation, ProductName,
                                                    Version, Edition, ServicePack, IsIntegratedSecurity, DataPath,
                                                    ServiceName, State, StartMode, [Clustered], InstanceID, InstanceName,
                                                    Sku, SqlServiceType, Environment, Isv)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      LEFT(d.fqdn, 128)                 AS SrcDeviceID,
      LEFT(i.Value + ''_'' + sst.Value + ''_'' + d.fqdn, 128) AS SrcID,
      ip.Value                          AS InstallLocation,
      dn.Value                          AS ProductName,
      v.Value                           AS Version,
      CASE
        WHEN ed.Value IS NOT NULL          THEN ed.Value
        WHEN sku.Value LIKE ''%Express%''    THEN ''Express''
        WHEN sku.Value LIKE ''%Enterprise%'' THEN ''Enterprise''
        WHEN sku.Value LIKE ''%Standard%''   THEN ''Standard''
        WHEN sku.Value LIKE ''%Developer%''  THEN ''Developer''
      END                               AS Edition,
      sp.Value                          AS ServicePack,
      CAST(NULL AS BIT)                 AS IsIntegratedSecurity,
      dp.Value                          AS DataPath,
      i.Value                           AS ServiceName,
      ss.Value                          AS State,
      CAST(NULL AS NVARCHAR(256))       AS StartMode,
      CAST(CASE
             WHEN c.Value = ''False'' THEN 0
             WHEN c.Value = ''True''  THEN 0
           END AS BIT)                  AS [Clustered],
      id.Value                          AS InstanceID,
      REPLACE(i.Value, ''MSSQL$'', '''')    AS InstanceName,
      sku.Value                         AS Sku,
      CAST(sst.Value AS INT)            AS SqlServiceType,
      CAST(NULL AS NVARCHAR(256))       AS Environment,
      CAST(NULL AS NVARCHAR(256))       AS Isv
    FROM f_HardwareProperties(''MGS_SqlService'') i
      INNER JOIN ' + @sTmpFQDN + ' d
        ON i.ComputerID = d.ComputerID
      INNER JOIN f_HardwareValuesByPropertyName(''DisplayName'') dn
        ON i.HardwareObjectID = dn.HardwareObjectID
          AND dn.Value NOT LIKE ''%Integration%'' AND dn.Value NOT LIKE ''%Full-text%Filter%''
          AND (dn.Value = ''SQL Server'' OR dn.Value LIKE ''SQL Server %'')
          AND (dn.Value NOT LIKE ''%Browser%'' AND dn.Value NOT LIKE ''%Agent%'' AND dn.Value NOT LIKE ''%Integration%'')
      LEFT JOIN f_HardwareValuesByPropertyName(''Version'') v
        ON i.HardwareObjectID = v.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''SKUNAME'') sku
        ON i.HardwareObjectID = sku.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''Edition'') ed
        ON i.HardwareObjectID = ed.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''INSTANCEID'') id
        ON i.HardwareObjectID = id.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''INSTALLPATH'') ip
        ON i.HardwareObjectID = ip.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''DATAPATH'') dp
        ON i.HardwareObjectID = dp.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''SQLServiceType'') sst
        ON i.HardwareObjectID = sst.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''CLUSTERED'') c
        ON i.HardwareObjectID = c.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''SPLEVEL'') sp
        ON i.HardwareObjectID = sp.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName(''SQLSTATES'') ss
        ON i.HardwareObjectID = ss.HardwareObjectID
    WHERE i.Property = ''Name'''
  EXECUTE (@sSQL)

  SET @sSQL = '
  DELETE x  -- Fix if same INstance with 2 different State
  FROM ' + @UniversalDB + '.[IN].[tblDatabaseServers] x
    INNER JOIN (SELECT
                    SrcDeviceID, 
                    SrcID, 
                    MAX([State]) AS [State]
                  FROM ' + @UniversalDB + '.[IN].[tblDatabaseServers] 
                  GROUP BY SrcDeviceId, SrcID, Sku 
                  HAVING COUNT([State]) > 1) tmp
       ON x.SrcDeviceId = tmp.SrcDeviceId
         AND x.SrcId = tmp.SrcId
         AND x.State = tmp.State'
  EXECUTE (@sSQL)

  SET @sSQL = '
  DELETE x -- Fix if same Instance with 2 diverend SKU (32bit u. 64bit)
  FROM ' + @UniversalDB + '.[IN].[tblDatabaseServers] x
    INNER JOIN (SELECT
                    SrcDeviceID, 
                    SrcID, 
                    MIN(Sku) AS Sku
                  FROM ' + @UniversalDB + '.[IN].[tblDatabaseServers] 
                  GROUP BY SrcDeviceId, SrcID, State 
                  HAVING COUNT([Sku]) > 1) tmp
       ON x.SrcDeviceId = tmp.SrcDeviceId
         AND x.SrcId = tmp.SrcId
         AND x.Sku = tmp.Sku'
  EXECUTE (@sSQL)

  /********************************************************************************************
   * Microsoft Exchange
   ********************************************************************************************
   * Description: Get all Exchange Server Information
   ********************************************************************************************
   * Created:     02.01.2019
   * Last Change:
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   * - InstallPath   Pfad der Installation
   * - ServicePack   Service Pack
   ********************************************************************************************/

  SET @sSQL = '
  INSERT INTO ' + @UniversalDB + '.[IN].[tblMailServers] (DataSourceID, SrcID, SrcDeviceID, ServerName, InstallPath, ProductName,
                                                    Version, Edition, Enterprise, ServicePack)
  SELECT DISTINCT
      ''' + @DataSourceID + '''                                           AS DataSourceID,
      LEFT(COALESCE(d.fqdn, CAST(d.ComputerID AS NVARCHAR(128))) 
             + exchange.Release + exchange.Edition, 128)                AS SrcID,
      LEFT(COALESCE(d.fqdn, CAST(d.ComputerID AS NVARCHAR(128))), 128)   AS SrcDeviceID,
      d.ComputerCN                                                       AS ServerName,
      CAST(NULL AS NVARCHAR(256))                                        AS InstallPath,
      ''Microsoft Exchange '' + exchange.Release + '' '' +exchange.Edition   AS ProductName,
      exchange.Version                                                   AS Version,
      exchange.Edition                                                   AS Edition,
      CASE
        WHEN exchange.Edition IS NULL THEN NULL
        WHEN exchange.Edition LIKE ''%Enterprise%'' THEN 1
        ELSE 0
      END                                                                AS Enterprise,
      CAST(NULL AS NVARCHAR(256))                                        AS ServicePack
    FROM (SELECT DISTINCT
              ex.ComputerID,
              ex.Value      AS ComputerName,
              exFQDN.Value  AS ComputerFQDN,
              exEd.Value    AS Edition,
              exV.Value     AS Version,
              CASE 
                WHEN exV.Value LIKE ''%15.1 %'' THEN ''2016''
                WHEN exV.Value LIKE ''%15.%''   THEN ''2013''
                WHEN exV.Value LIKE ''%14.%''   THEN ''2010''
                WHEN exV.Value LIKE ''%8.%''    THEN ''2007''
                WHEN exV.Value LIKE ''%6.5 %''  THEN ''2003''
                WHEN exV.Value LIKE ''%6.0 %''  THEN ''2000''
                ELSE NULL
              END           AS Release
            FROM f_HardwareProperties(''MGS_ExchangeServer'') ex
              LEFT JOIN f_HardwareValuesByPropertyName(''fqdn'') exFQDN
                ON ex.HardwareObjectID = exFQDN.HardwareObjectID
              LEFT JOIN f_HardwareValuesByPropertyName(''Edition'') exEd
                ON ex.HardwareObjectID = exED.HardwareObjectID
              LEFT JOIN f_HardwareValuesByPropertyName(''AdminDisplayVersion'') exV
                ON ex.HardwareObjectID = exV.HardwareObjectID
            WHERE ex.Property = ''Name'') exchange
      INNER JOIN ' + @sTmpFQDN + ' d
        ON exchange.ComputerID = d.ComputerID'
  EXECUTE (@sSQL)


  /********************************************************************************************
   * Start Impport
   ********************************************************************************************
   * Description: Start the Import to the Universal Inventory
   ********************************************************************************************
   * Created:     12.12.2018
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/
  SET @sSQL = '
  DECLARE @lastErrorMsg NVARCHAR(4000) = ' + IIF(@lastErrorMsg IS NULL, 'NULL',  '''' + @lastErrorMsg + '''') + '
  EXEC ' + @UniversalDB + '.[in].[sp_Import]  @lastErrorMsg
  SELECT @lastErrorMsg AS [@lastErrorMsg]'
EXECUTE (@sSQL)

  /********************************************************************************************
   * Delete Temorary Tables
   ********************************************************************************************
   * Description: Delete all Temporary Tables
   ********************************************************************************************
   * Created:     14.12.2018
   * Last Change: 
   * Author:      p.arbogast@raynet.de
   ********************************************************************************************
   * Open Points:
   ********************************************************************************************/
  SET @sSQL = 'DROP TABLE ' + @sTmpFQDN
  EXECUTE (@sSQL)
END
GO